package  
{
	import flash.display.BlendMode;
	import flash.geom.Point;
	import net.flashpunk.Entity;
	import net.flashpunk.FP;
	import net.flashpunk.graphics.Image;
	import net.flashpunk.utils.Input;
	import net.flashpunk.utils.Key;
	import net.flashpunk.World;
	import punk.fx.effects.BlurFX;
	import punk.fx.effects.ColorTransformFX;
	import punk.fx.effects.DynaLightFX;
	import punk.fx.effects.PBRadialFocusFX;
	import punk.fx.effects.ScanLinesFX;
	import punk.fx.graphics.FXImage;
	import punk.fx.graphics.FXLayer;
	
	/**
	 * ...
	 * @author azrafe7
	 */
	public class GameWorld extends World 
	{
		[Embed(source = "assets/background.png")]
		protected var BACKGROUND:Class;
		
		[Embed(source = "assets/cave.png")]
		protected var CAVE:Class;
		
		protected var backImage:Image;
		protected var caveImage:Image;
		
		protected var backEntity:Entity;
		protected var caveEntity:Entity;
		protected var platformEntity:Platform;

		protected var occludersLayer:FXLayer;
		protected var dynaLightsImage:FXImage;
		protected var dynaLightFX:DynaLightFX;
		protected var dynaLightFX2:DynaLightFX;
		
		private var radialFX:PBRadialFocusFX = new PBRadialFocusFX(0,0,.5,7);
		private var blurFX:BlurFX = new BlurFX(3);
		private var redColorFX:ColorTransformFX = new ColorTransformFX(2.3, 1.2, .3, 1, 30, 30, 50);
		private var blueColorFX:ColorTransformFX = new ColorTransformFX(.3, .5, 1.5, 1);
		private var greenColorFX:ColorTransformFX = new ColorTransformFX(.3, 1.5, .5, 1);
		
		protected var mousePos:Point = new Point;
		
		public function GameWorld() 
		{
			backImage = new Image(BACKGROUND);
			backEntity = new Entity(0, 0, backImage);
			add(backEntity);

			platformEntity = new Platform(130, 260, 130, 160);
			platformEntity.visible = false;
			add(platformEntity);
			
			var platformEntity2:Platform = new Platform(190, 220, 190, 100, 3);
			platformEntity2.visible = false;
			add(platformEntity2);
			
			caveImage = new Image(CAVE);
			caveEntity = new Entity(0, 0, caveImage);
			caveEntity.visible = false;
			add(caveEntity);
			
			occludersLayer = new FXLayer(FP.width/FP.screen.scale, FP.height/FP.screen.scale);
			occludersLayer.entities.add([caveEntity, platformEntity, platformEntity2]);
			addGraphic(occludersLayer, -1);
			
			dynaLightsImage = new FXImage(occludersLayer.buffer.clone());
			dynaLightFX = new DynaLightFX(occludersLayer.getSource(), .5);
			dynaLightFX2 = new DynaLightFX(occludersLayer.getSource(), .5);
			
			setupDynaLights();
			
			dynaLightsImage.effects.add([dynaLightFX, dynaLightFX2]);
			dynaLightsImage.blend = BlendMode.MULTIPLY;
			addGraphic(dynaLightsImage, -1);
		}
		
		protected function setupDynaLights():void 
		{
			// first set of lights (blue)
			dynaLightFX.postEffects.add([blurFX, blueColorFX]);
			//dynaLightFX.active = false;
			dynaLightFX.lights[0].setProps({x:220, y: 50});
			dynaLightFX.lights[0].setFallOff(-.05, .05, 200);
			
			// second set of lights (red + ambience)
			dynaLightFX2.postEffects.add([blurFX, redColorFX/*, new ScanLinesFX(true, 10)*/]);
			//dynaLightFX2.active = false;
			dynaLightFX2.lights[0].setFallOff(-.05, .05, 200);
			dynaLightFX2.lights[0].penetration = 0.0;

			
			dynaLightFX2.lights[1].active = true;
			dynaLightFX2.lights[1].x = 150;
			dynaLightFX2.lights[1].y = 150;
			dynaLightFX2.lights[1].setFallOff(-.05, .05, 11180);
			
			//dynaLightFX2.lights[2].active = true;
			dynaLightFX2.lights[2].x = 30;
			dynaLightFX2.lights[2].y = 30;
			dynaLightFX2.lights[2].setFallOff( -.05, .05, 180);
			
			//Input.mouseCursor = "hide";
		}
		
		override public function update():void 
		{
			super.update();
			//Telemetry.sendMetric("this", 0);
			dynaLightFX.lights[0].trackedObj = Input.check(Key.CONTROL) && !Input.check(Key.SHIFT) ? mousePos : null;
			dynaLightFX2.lights[0].trackedObj = !Input.check(Key.CONTROL) && !Input.check(Key.SHIFT) ? mousePos : null;
			dynaLightFX.lights[2].trackedObj = Input.check(Key.SHIFT) ? mousePos : null;
			mousePos.x = Input.mouseX;
			mousePos.y = Input.mouseY;
			
			/*
			radialFX.centerX = mousePos.x * .5;
			radialFX.centerY = mousePos.y * .5;
			*/
			
			//trace(dynaLightFX.lights[2].active);
		}
	}

}